/*
 * by Stephan Baerwolf, matrixstorm@gmx.de
 * 
 * NOT FOR COMMERCIAL USE
 * 
 */

#include "vusbdevice.h"

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <time.h>

#include <glib.h>
#include <gtk/gtk.h>


#define USBDEV_SHARED_VENDOR    0x16C0  /* VOTI */
#define USBDEV_SHARED_PRODUCT   0x05DC  /* Obdev's free shared PID */
/* Use obdev's generic shared VID/PID pair and follow the rules outlined
 * in firmware/usbdrv/USBID-License.txt.
 */

#define USBDEV_VENDOR_NAME	"www.matrixstorm.com"
#define USBDEV_DEVICE_NAME	"AVR<->PC"
#define USBDEV_SERIAL		"201308202244"

/* USB Request-Commands */
#define PSCMD_CALL_ECHO		0
#define PSCMD_CALL_GETSTATUS	1
#define PSCMD_CALL_SETSTATUS	2

/* USB Response-Codes */
#define PSCMD_RETN_OKAY		(0x0)
#define PSCMD_RETN_BUSY		(0x1)
#define PSCMD_RETN_UNKNOW	(0xff)


static gboolean		need_report	= TRUE;
static usb_dev_handle	*handle		= NULL;
static GtkBuilder	*builder	= NULL;

#define MYDEBUG(msg...) ({fprintf(stderr,"%s:%i (%s)   ",__FILE__,__LINE__, __FUNCTION__); fprintf(stderr, ##msg); })

gboolean refresh_color_timer(GtkColorSelection *colorselection) {
  if (handle) {
    unsigned char       buffer[8 + 1 + 1];
    GdkColor		color, old_color;
    int                 nBytes;

    nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, PSCMD_CALL_GETSTATUS, 0, 0, (char *)buffer, sizeof(buffer), 5000);
    if (nBytes >= 8) {
      if (buffer[1]==PSCMD_RETN_OKAY) {
	color.red  = ((guint16)buffer[3]<<8) | (guint16)buffer[2];
	color.green= ((guint16)buffer[5]<<8) | (guint16)buffer[4];
	color.blue = ((guint16)buffer[7]<<8) | (guint16)buffer[6];
      }
    }
    gtk_color_selection_get_current_color(colorselection, &old_color);
    if ((old_color.red!=color.red) || (old_color.green!=color.green) || (old_color.blue!=color.blue)) {
      need_report = FALSE;
      gtk_color_selection_set_current_color(colorselection, &color);
    }
    return TRUE;
  }
  
  return FALSE;
}

G_MODULE_EXPORT void on_colorselection1_color_changed(GtkWidget *widget, gpointer data) {
  GdkColor color;
  GtkColorSelection *self = GTK_COLOR_SELECTION (widget);
  
  if (need_report) {
    gtk_color_selection_get_current_color(self, &color);
    if (handle) {
      unsigned char buffer[6];
      buffer[0] = color.red   & 0xff;
      buffer[1] = color.red   >> 0x8;
      buffer[2] = color.green & 0xff;
      buffer[3] = color.green >> 0x8;
      buffer[4] = color.blue  & 0xff;
      buffer[5] = color.blue  >> 0x8;
      usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT, PSCMD_CALL_SETSTATUS, 0, 0, buffer, sizeof(buffer), 5000);
    } else MYDEBUG("here: red=%3i, green=%3i, blue=%3i\n", color.red, color.green, color.blue);
  }
  need_report = TRUE;
}

G_MODULE_EXPORT void on_window1_destroy(GtkWidget *widget, gpointer data) {
  gtk_main_quit ();
}

int main (int argc, char *argv[]) {
  srand(time(NULL));
  vusbdevice_initialize();
  
  gtk_init(&argc, &argv);

  if (g_module_supported()) {
    builder = gtk_builder_new ();
    if (builder) {
      extern const gchar	colordialog[]		asm("_binary_colordialog_glade_start");
      extern const void		colordialog_size	asm("_binary_colordialog_glade_size");
      if (gtk_builder_add_from_string (builder, colordialog, (gsize)&colordialog_size, NULL)) {
	GtkWidget  *window1 = GTK_WIDGET (gtk_builder_get_object (builder, "window1"));;
  
	{
	  GdkColor color;
	  GtkColorSelection *colorselection1 = GTK_COLOR_SELECTION(gtk_builder_get_object (builder, "colorselection1"));

	  color.red  = 0;
	  color.green= 0;
	  color.blue = 0;
	  gtk_color_selection_set_current_color(colorselection1, &color);

	  if(usbOpenDevice(&handle, USBDEV_SHARED_VENDOR, USBDEV_SHARED_PRODUCT, USBDEV_VENDOR_NAME, USBDEV_DEVICE_NAME, USBDEV_SERIAL) != 0) {
	    if (handle) usb_close(handle);
	    handle=NULL;
	  }

	  refresh_color_timer(colorselection1);
	  g_timeout_add_full(G_PRIORITY_LOW, 100, (GSourceFunc) refresh_color_timer, (gpointer) colorselection1, NULL);
	}
	
	gtk_builder_connect_signals (builder, NULL);                 
	gtk_widget_show_all (window1);

	gtk_main ();
      } else MYDEBUG("reading file failed\n");
    } else MYDEBUG("out of memory?\n");
  } else MYDEBUG("g_module is NOT supported\n");

  if (builder) g_object_unref (G_OBJECT (builder));
  builder=NULL;
  if (handle) usb_close(handle);
  handle=NULL;
  vusbdevice_finalize();
  return 0;
}
