
#include "vusbdevice.h"
#include  <string.h>

static long initialize_count = 0;

int vusbdevice_initialize(void) {
  if (initialize_count==0L) {
    usb_init();
  }
  initialize_count++;
  return 0;
}

int vusbdevice_finalize(void) {
  if (initialize_count>0) {
    initialize_count--;
  } else return USB_ERROR_NOINIT;
  return 0;
}





int usbGetStringAscii(usb_dev_handle *dev, int index, int langid, char *buf, int buflen) {
  unsigned char    buffer[256];
  int     rval, i;

  if (initialize_count<=0L) return USB_ERROR_NOINIT;

  rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, langid, buffer, sizeof(buffer), 1000);
  if (rval < 0)	{
    return rval;
  }

  if (buffer[1] != USB_DT_STRING) {
    return 0;
  }

  if (buffer[0] < rval) {
    rval = buffer[0];
  }

  /* lossy conversion to ISO Latin1 */
  rval >>= 1;
  for (i=1;i<rval;i++){
    if (i > buflen) {
      /* destination buffer overflow */
      break;
    }
    buf[i-1] = buffer[i<<1];
    if (buffer[(i<<1) + 1] != 0) {
      /* outside of ISO Latin1 range */
      buf[i-1] = '?';
    }
  }
  buf[i-1] = 0;
  return i-1;
}

int usbOpenDevice(usb_dev_handle **device, int vendor, int product, char *vendorName, char *productName, char *serial) {
  struct usb_bus      *bus;
  struct usb_device   *dev;
  usb_dev_handle      *handle = NULL;
  int                 errorCode = USB_ERROR_NOTFOUND;

  if (initialize_count<=0L) return USB_ERROR_NOINIT;

  usb_find_busses();
  usb_find_devices();
  for(bus=usb_get_busses(); bus; bus=bus->next) {
    for(dev=bus->devices; dev; dev=dev->next) {
      if(dev->descriptor.idVendor == vendor && dev->descriptor.idProduct == product) {
	char    string[256];
	int     len;

	/* we need to open the device in order to query strings */
	handle = usb_open(dev);
	if(!handle) {
	  errorCode = USB_ERROR_ACCESS;
	  continue;
	}

	if(vendorName == NULL && productName == NULL) {
	  /* name does not matter */
	  break;
	}

	/* now check whether the names match: */
	len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
	if (len < 0) {
	  errorCode = USB_ERROR_IO;
	} else {
	  errorCode = USB_ERROR_NOTFOUND;
	  if (strcmp(string, vendorName) == 0) {
	    len = usbGetStringAscii(handle, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
	    if (len < 0) {
	      errorCode = USB_ERROR_IO;
	    } else {
	      errorCode = USB_ERROR_NOTFOUND;
	      if (strcmp(string, productName) == 0) {
		len = usbGetStringAscii(handle, dev->descriptor.iSerialNumber, 0x0409, string, sizeof(string));
		if(len < 0) {
		  errorCode = USB_ERROR_IO;
		} else {
		  errorCode = USB_ERROR_NOTFOUND;
		  if(strcmp(string, serial) == 0) {
		    break;
		  }
		}
	      }
	    }
	  }
	}
	usb_close(handle);
	handle = NULL;
      }
    }
    if (handle) {
      break;
    }
  }
  if (handle != NULL) {
    errorCode = 0;
    *device = handle;
  }
  return errorCode;
}
